const Metadata = require("../../metadata");
const BoostProvider = require("./../provider");
const BoostContext = require("./../context");

const ConsumerManager = {
    observeMap: {},
    watchMap: {},
    set(clazz) {
        let fields = Metadata.getMethodMetadata(clazz.prototype);
        Reflect.ownKeys(fields).filter(propName => !!fields[propName].consumer).forEach(propName => {
            let config = fields[propName].consumer, { name, type, method = 'observe' } = config;
            let map = this.observeMap;
            if (method !== 'observe') {
                map = this.watchMap;
            }
            if (!map[name]) {
                map[name] = [];
            }
            if (!map[name].find(a => a.name === name && a.type === type)) {
                map[name].push({
                    name,
                    type,
                    method,
                    config,
                    target: clazz,
                    propName
                });
            };
        });
    },
    getObserveNames() {
        return Reflect.ownKeys(this.observeMap);
    },
    getWatchNames() {
        return Reflect.ownKeys(this.watchMap);
    },
    excuteObserve(name, info) {
        let { type } = info;
        let _target = (this.observeMap[name] || []).find(a => a.name === name && a.type === type);
        if (_target) {
            let { propName, target } = _target;
            let targetObject = BoostProvider.getEntity(target);
            return Promise.resolve().then(() => targetObject.beforeExcute()).then(result => {
                if (result !== false) {
                    return targetObject[propName](Object.assign({}, BoostContext.getContextInfo().server.context, { message: info }));
                }
            }).then(() => targetObject.afterExcute());
        }
    },
    excuteWatch(name, info) {
        (this.watchMap[name] || []).forEach(({ target, propName }) => {
            let targetObject = BoostProvider.getEntity(target);
            return Promise.resolve().then(() => targetObject.beforeExcute()).then(result => {
                if (result !== false) {
                    return targetObject[propName](Object.assign({}, BoostContext.getContextInfo().server.context, { message: info }));
                }
            }).then(() => targetObject.afterExcute());
        });
    }
};

module.exports = ConsumerManager;